@@ -2,43 +2,34 @@ package m
 
 import (
 	"fmt"
-	"math"
 	"regexp"
 	"strings"
 	"time"
 
 	"github.com/alecthomas/chroma/v2"
 	log "github.com/sirupsen/logrus"
+	"github.com/walles/moar/m/linenumbers"
+	"github.com/walles/moar/m/textstyles"
 	"github.com/walles/moar/twin"
 )
 
-type _PagerMode int
-
-const (
-	_Viewing _PagerMode = iota
-	_Searching
-	_NotFound
-	_GotoLine
-)
+type PagerMode interface {
+	onKey(key twin.KeyCode)
+	onRune(char rune)
+	drawFooter(statusText string, spinner string)
+}
 
 type StatusBarOption int
 
 const (
+	//revive:disable-next-line:var-naming
 	STATUSBAR_STYLE_INVERSE StatusBarOption = iota
+	//revive:disable-next-line:var-naming
 	STATUSBAR_STYLE_PLAIN
+	//revive:disable-next-line:var-naming
 	STATUSBAR_STYLE_BOLD
 )
 
-// How do we render unprintable characters?
-type UnprintableStyle int
-
-const (
-	UNPRINTABLE_STYLE_HIGHLIGHT UnprintableStyle = iota
-	UNPRINTABLE_STYLE_WHITESPACE
-)
-
-var unprintableStyle UnprintableStyle
-
 type eventSpinnerUpdate struct {
 	spinner string
 }
@@ -57,13 +48,17 @@ type Pager struct {
 	scrollPosition      scrollPosition
 	leftColumnZeroBased int
 
-	mode           _PagerMode
+	// Maybe this should be renamed to "controller"? Because it controls the UI?
+	// But since we replace it in a lot of places based on the UI mode, maybe
+	// mode is better?
+	mode PagerMode
+
 	searchString   string
 	searchPattern  *regexp.Regexp
 	gotoLineString string
 
 	// We used to have a "Following" field here. If you want to follow, set
-	// TargetLineNumberOneBased to math.MaxInt instead, see below.
+	// TargetLineNumber to LineNumberMax() instead, see below.
 
 	isShowingHelp bool
 	preHelpState  *_PreHelpState
@@ -74,7 +69,7 @@ type Pager struct {
 	StatusBarStyle StatusBarOption
 	ShowStatusBar  bool
 
-	UnprintableStyle UnprintableStyle
+	UnprintableStyle textstyles.UnprintableStyleT
 
 	WrapLongLines bool
 
@@ -87,9 +82,9 @@ type Pager struct {
 
 	SideScrollAmount int // Should be positive
 
-	// If non-zero, scroll to this line number as soon as possible. Set to
-	// math.MaxInt to follow the end of the input (tail).
-	TargetLineNumberOneBased int
+	// If non-nil, scroll to this line number as soon as possible. Set this
+	// value to LineNumberMax() to follow the end of the input (tail).
+	TargetLineNumber *linenumbers.LineNumber
 
 	// If true, pager will clear the screen on return. If false, pager will
 	// clear the last line, and show the cursor.
@@ -102,13 +97,18 @@ type Pager struct {
 
 	// Length of the longest line displayed. This is used for limiting scrolling to the right.
 	longestLineLength int
+
+	// Bookmarks that you can come back to.
+	//
+	// Ref: https://github.com/walles/moar/issues/175
+	marks map[rune]scrollPosition
 }
 
 type _PreHelpState struct {
-	reader                   *Reader
-	scrollPosition           scrollPosition
-	leftColumnZeroBased      int
-	targetLineNumberOneBased int
+	reader              *Reader
+	scrollPosition      scrollPosition
+	leftColumnZeroBased int
+	targetLineNumber    *linenumbers.LineNumber
 }
 
 const _EofMarkerFormat = "\x1b[7m" // Reverse video
@@ -127,12 +127,14 @@ Moving around
 * Arrow keys
 * Alt key plus left / right arrow steps one column at a time
 * Left / right can be used to hide / show line numbers
+* Home and End for start / end of the document
+* 'g' for going to a specific line number
+* 'm' sets a mark, you will be asked for a letter to label it with
+* ' (single quote) jumps to the mark
 * CTRL-p moves to the previous line
 * CTRL-n moves to the next line
-* 'g' for going to a specific line number
 * PageUp / 'b' and PageDown / 'f'
 * SPACE moves down a page
-* Home and End for start / end of the document
 * < / 'gg' to go to the start of the document
 * > / 'G' to go to the end of the document
 * 'h', 'l' for left and right (as in vim)
@@ -163,10 +165,6 @@ Source Code
 Available at https://github.com/walles/moar/.
 `)
 
-func (pm _PagerMode) isViewing() bool {
-	return pm == _Viewing || pm == _NotFound
-}
-
 // NewPager creates a new Pager with default settings
 func NewPager(r *Reader) *Pager {
 	var name string
@@ -175,7 +173,8 @@ func NewPager(r *Reader) *Pager {
 	} else {
 		name = "Pager " + *r.name
 	}
-	return &Pager{
+
+	pager := Pager{
 		reader:           r,
 		quit:             false,
 		ShowLineNumbers:  true,
@@ -186,6 +185,10 @@ func NewPager(r *Reader) *Pager {
 		ScrollRightHint:  twin.NewCell('>', twin.StyleDefault.WithAttr(twin.AttrReverse)),
 		scrollPosition:   newScrollPosition(name),
 	}
+
+	pager.mode = PagerModeViewing{pager: &pager}
+
+	return &pager
 }
 
 // How many lines are visible on screen? Depends on screen height and whether or
@@ -198,6 +201,7 @@ func (p *Pager) visibleHeight() int {
 	return height
 }
 
+// Draw the footer string at the bottom using the status bar style
 func (p *Pager) setFooter(footer string) {
 	width, height := p.screen.Size()
 
@@ -224,7 +228,7 @@ func (p *Pager) Quit() {
 	p.reader = p.preHelpState.reader
 	p.scrollPosition = p.preHelpState.scrollPosition
 	p.leftColumnZeroBased = p.preHelpState.leftColumnZeroBased
-	p.TargetLineNumberOneBased = p.preHelpState.targetLineNumberOneBased
+	p.TargetLineNumber = p.preHelpState.targetLineNumber
 	p.preHelpState = nil
 }
 
@@ -255,183 +259,15 @@ func (p *Pager) moveRight(delta int) {
 }
 
 func (p *Pager) handleScrolledUp() {
-	p.TargetLineNumberOneBased = 0
+	p.TargetLineNumber = nil
 }
 
 func (p *Pager) handleScrolledDown() {
 	if p.isScrolledToEnd() {
-		p.TargetLineNumberOneBased = math.MaxInt
+		reallyHigh := linenumbers.LineNumberMax()
+		p.TargetLineNumber = &reallyHigh
 	} else {
-		p.TargetLineNumberOneBased = 0
-	}
-}
-
-func (p *Pager) onKey(keyCode twin.KeyCode) {
-	if p.mode == _Searching {
-		p.onSearchKey(keyCode)
-		return
-	}
-	if p.mode == _GotoLine {
-		p.onGotoLineKey(keyCode)
-		return
-	}
-	if p.mode != _Viewing && p.mode != _NotFound {
-		panic(fmt.Sprint("Unhandled mode: ", p.mode))
-	}
-
-	// Reset the not-found marker on non-search keypresses
-	p.mode = _Viewing
-
-	switch keyCode {
-	case twin.KeyEscape:
-		p.Quit()
-
-	case twin.KeyUp:
-		// Clipping is done in _Redraw()
-		p.scrollPosition = p.scrollPosition.PreviousLine(1)
-		p.handleScrolledUp()
-
-	case twin.KeyDown, twin.KeyEnter:
-		// Clipping is done in _Redraw()
-		p.scrollPosition = p.scrollPosition.NextLine(1)
-		p.handleScrolledDown()
-
-	case twin.KeyRight:
-		p.moveRight(p.SideScrollAmount)
-
-	case twin.KeyLeft:
-		p.moveRight(-p.SideScrollAmount)
-
-	case twin.KeyAltRight:
-		p.moveRight(1)
-
-	case twin.KeyAltLeft:
-		p.moveRight(-1)
-
-	case twin.KeyHome:
-		p.scrollPosition = newScrollPosition("Pager scroll position")
-		p.handleScrolledUp()
-
-	case twin.KeyEnd:
-		p.scrollToEnd()
-
-	case twin.KeyPgUp:
-		p.scrollPosition = p.scrollPosition.PreviousLine(p.visibleHeight())
-		p.handleScrolledUp()
-
-	case twin.KeyPgDown:
-		p.scrollPosition = p.scrollPosition.NextLine(p.visibleHeight())
-		p.handleScrolledDown()
-
-	default:
-		log.Debugf("Unhandled key event %v", keyCode)
-	}
-}
-
-func (p *Pager) onRune(char rune) {
-	if p.mode == _Searching {
-		p.onSearchRune(char)
-		return
-	}
-	if p.mode == _GotoLine {
-		p.onGotoLineRune(char)
-		return
-	}
-	if p.mode != _Viewing && p.mode != _NotFound {
-		panic(fmt.Sprint("Unhandled mode: ", p.mode))
-	}
-
-	switch char {
-	case 'q':
-		p.Quit()
-
-	case '?':
-		if !p.isShowingHelp {
-			p.preHelpState = &_PreHelpState{
-				reader:                   p.reader,
-				scrollPosition:           p.scrollPosition,
-				leftColumnZeroBased:      p.leftColumnZeroBased,
-				targetLineNumberOneBased: p.TargetLineNumberOneBased,
-			}
-			p.reader = _HelpReader
-			p.scrollPosition = newScrollPosition("Pager scroll position")
-			p.leftColumnZeroBased = 0
-			p.TargetLineNumberOneBased = 0
-			p.isShowingHelp = true
-		}
-
-	case '=':
-		p.ShowStatusBar = !p.ShowStatusBar
-
-	// '\x10' = CTRL-p, should scroll up one line.
-	// Ref: https://github.com/walles/moar/issues/107#issuecomment-1328354080
-	case 'k', 'y', '\x10':
-		// Clipping is done in _Redraw()
-		p.scrollPosition = p.scrollPosition.PreviousLine(1)
-		p.handleScrolledUp()
-
-	// '\x0e' = CTRL-n, should scroll down one line.
-	// Ref: https://github.com/walles/moar/issues/107#issuecomment-1328354080
-	case 'j', 'e', '\x0e':
-		// Clipping is done in _Redraw()
-		p.scrollPosition = p.scrollPosition.NextLine(1)
-		p.handleScrolledDown()
-
-	case 'l':
-		// vim right
-		p.moveRight(p.SideScrollAmount)
-
-	case 'h':
-		// vim left
-		p.moveRight(-p.SideScrollAmount)
-
-	case '<':
-		p.scrollPosition = newScrollPosition("Pager scroll position")
-		p.handleScrolledUp()
-
-	case '>', 'G':
-		p.scrollToEnd()
-
-	case 'f', ' ':
-		p.scrollPosition = p.scrollPosition.NextLine(p.visibleHeight())
-		p.handleScrolledDown()
-
-	case 'b':
-		p.scrollPosition = p.scrollPosition.PreviousLine(p.visibleHeight())
-		p.handleScrolledUp()
-
-	// '\x15' = CTRL-u, should work like just 'u'.
-	// Ref: https://github.com/walles/moar/issues/90
-	case 'u', '\x15':
-		p.scrollPosition = p.scrollPosition.PreviousLine(p.visibleHeight() / 2)
-		p.handleScrolledUp()
-
-	// '\x04' = CTRL-d, should work like just 'd'.
-	// Ref: https://github.com/walles/moar/issues/90
-	case 'd', '\x04':
-		p.scrollPosition = p.scrollPosition.NextLine(p.visibleHeight() / 2)
-		p.handleScrolledDown()
-
-	case '/':
-		p.mode = _Searching
-		p.searchString = ""
-		p.searchPattern = nil
-
-	case 'g':
-		p.mode = _GotoLine
-		p.gotoLineString = ""
-
-	case 'n':
-		p.scrollToNextSearchHit()
-
-	case 'p', 'N':
-		p.scrollToPreviousSearchHit()
-
-	case 'w':
-		p.WrapLongLines = !p.WrapLongLines
-
-	default:
-		log.Debugf("Unhandled rune keypress '%s'/0x%08x", string(char), int32(char))
+		p.TargetLineNumber = &linenumbers.LineNumber{}
 	}
 }
 
@@ -473,12 +309,14 @@ func (p *Pager) StartPaging(screen twin.Screen, chromaStyle *chroma.Style, chrom
 		}
 	}()
 
-	unprintableStyle = p.UnprintableStyle
+	textstyles.UnprintableStyle = p.UnprintableStyle
 	consumeLessTermcapEnvs(chromaStyle, chromaFormatter)
-	styleUi(chromaStyle, chromaFormatter, p.StatusBarStyle)
+	styleUI(chromaStyle, chromaFormatter, p.StatusBarStyle)
 
 	p.screen = screen
 	p.linePrefix = getLineColorPrefix(chromaStyle, chromaFormatter)
+	p.mode = PagerModeViewing{pager: p}
+	p.marks = make(map[rune]scrollPosition)
 
 	go func() {
 		for range p.reader.moreLinesAdded {
@@ -554,11 +392,11 @@ func (p *Pager) StartPaging(screen twin.Screen, chromaStyle *chroma.Style, chrom
 		switch event := event.(type) {
 		case twin.EventKeyCode:
 			log.Tracef("Handling key event %d...", event.KeyCode())
-			p.onKey(event.KeyCode())
+			p.mode.onKey(event.KeyCode())
 
 		case twin.EventRune:
 			log.Tracef("Handling rune event '%c'/0x%04x...", event.Rune(), event.Rune())
-			p.onRune(event.Rune())
+			p.mode.onRune(event.Rune())
 
 		case twin.EventMouse:
 			log.Tracef("Handling mouse event %d...", event.Buttons())
@@ -586,14 +424,15 @@ func (p *Pager) StartPaging(screen twin.Screen, chromaStyle *chroma.Style, chrom
 			return
 
 		case eventMoreLinesAvailable:
-			if p.mode.isViewing() && p.TargetLineNumberOneBased > 0 {
+			if p.TargetLineNumber != nil {
 				// The user wants to scroll down to a specific line number
-				if p.reader.GetLineCount() >= p.TargetLineNumberOneBased {
-					p.scrollPosition = NewScrollPositionFromLineNumberOneBased(p.TargetLineNumberOneBased, "goToTargetLineNumber")
-					p.TargetLineNumberOneBased = 0
-				} else {
+				if linenumbers.LineNumberFromLength(p.reader.GetLineCount()).IsBefore(*p.TargetLineNumber) {
 					// Not there yet, keep scrolling
 					p.scrollToEnd()
+				} else {
+					// We see the target, scroll to it
+					p.scrollPosition = NewScrollPositionFromLineNumber(*p.TargetLineNumber, "goToTargetLineNumber")
+					p.TargetLineNumber = nil
 				}
 			}
 
