@@ -8,18 +8,24 @@ import (
 	"strings"
 	"testing"
 
+	"github.com/alecthomas/chroma/v2"
 	"github.com/alecthomas/chroma/v2/formatters"
+	"github.com/alecthomas/chroma/v2/lexers"
 	"github.com/alecthomas/chroma/v2/styles"
 	"github.com/google/go-cmp/cmp"
+	"github.com/walles/moar/m/linenumbers"
+	"github.com/walles/moar/m/textstyles"
 	"github.com/walles/moar/twin"
 	"gotest.tools/v3/assert"
 )
 
+//revive:disable:empty-block
+
 const blueBackgroundClearToEol0 = "\x1b[44m\x1b[0K" // With 0 before the K, should clear to EOL
 const blueBackgroundClearToEol = "\x1b[44m\x1b[K"   // No 0 before the K, should also clear to EOL
 
 func TestUnicodeRendering(t *testing.T) {
-	reader := NewReaderFromStream("", strings.NewReader("åäö"))
+	reader := NewReaderFromText("", "åäö")
 
 	var answers = []twin.Cell{
 		twin.NewCell('å', twin.StyleDefault),
@@ -42,8 +48,8 @@ func assertCellsEqual(t *testing.T, expected twin.Cell, actual twin.Cell) {
 }
 
 func TestFgColorRendering(t *testing.T) {
-	reader := NewReaderFromStream("", strings.NewReader(
-		"\x1b[30ma\x1b[31mb\x1b[32mc\x1b[33md\x1b[34me\x1b[35mf\x1b[36mg\x1b[37mh\x1b[0mi"))
+	reader := NewReaderFromText("",
+		"\x1b[30ma\x1b[31mb\x1b[32mc\x1b[33md\x1b[34me\x1b[35mf\x1b[36mg\x1b[37mh\x1b[0mi")
 
 	var answers = []twin.Cell{
 		twin.NewCell('a', twin.StyleDefault.WithForeground(twin.NewColor16(0))),
@@ -64,7 +70,7 @@ func TestFgColorRendering(t *testing.T) {
 }
 
 func TestPageEmpty(t *testing.T) {
-	reader := NewReaderFromStream("", strings.NewReader(""))
+	reader := NewReaderFromText("", "")
 
 	firstRowCells := startPaging(t, reader).GetRow(0)
 
@@ -74,7 +80,7 @@ func TestPageEmpty(t *testing.T) {
 
 func TestBrokenUtf8(t *testing.T) {
 	// The broken UTF8 character in the middle is based on "©" = 0xc2a9
-	reader := NewReaderFromStream("", strings.NewReader("abc\xc2def"))
+	reader := NewReaderFromText("", "abc\xc2def")
 
 	var answers = []twin.Cell{
 		twin.NewCell('a', twin.StyleDefault),
@@ -116,7 +122,7 @@ func startPaging(t *testing.T, reader *Reader) *twin.FakeScreen {
 
 // assertIndexOfFirstX verifies the (zero-based) index of the first 'x'
 func assertIndexOfFirstX(t *testing.T, s string, expectedIndex int) {
-	reader := NewReaderFromStream("", strings.NewReader(s))
+	reader := NewReaderFromText("", s)
 
 	contents := startPaging(t, reader).GetRow(0)
 	for pos, cell := range contents {
@@ -163,7 +169,7 @@ func TestCodeHighlighting(t *testing.T) {
 		panic("Getting current filename failed")
 	}
 
-	reader, err := NewReaderFromFilename(filename, *styles.Get("native"), formatters.TTY16m)
+	reader, err := NewReaderFromFilename(filename, *styles.Get("native"), formatters.TTY16m, nil)
 	if err != nil {
 		panic(err)
 	}
@@ -204,12 +210,12 @@ func TestUnicodePrivateUse(t *testing.T) {
 }
 
 func resetManPageFormat() {
-	manPageBold = twin.StyleDefault.WithAttr(twin.AttrBold)
-	manPageUnderline = twin.StyleDefault.WithAttr(twin.AttrUnderline)
+	textstyles.ManPageBold = twin.StyleDefault.WithAttr(twin.AttrBold)
+	textstyles.ManPageUnderline = twin.StyleDefault.WithAttr(twin.AttrUnderline)
 }
 
 func testManPageFormatting(t *testing.T, input string, expected twin.Cell) {
-	reader := NewReaderFromStream("", strings.NewReader(input))
+	reader := NewReaderFromText("", input)
 
 	// Without these lines the man page tests will fail if either of these
 	// environment variables are set when the tests are run.
@@ -262,7 +268,7 @@ func TestToPattern(t *testing.T) {
 }
 
 func TestFindFirstHitSimple(t *testing.T) {
-	reader := NewReaderFromStream("TestFindFirstHitSimple", strings.NewReader("AB"))
+	reader := NewReaderFromText("TestFindFirstHitSimple", "AB")
 	pager := NewPager(reader)
 	pager.screen = twin.NewFakeScreen(40, 10)
 
@@ -273,12 +279,12 @@ func TestFindFirstHitSimple(t *testing.T) {
 	pager.searchPattern = toPattern("AB")
 
 	hit := pager.findFirstHit(newScrollPosition("TestFindFirstHitSimple"), false)
-	assert.Equal(t, hit.internalDontTouch.lineNumberOneBased, 1)
+	assert.Assert(t, hit.internalDontTouch.lineNumber.IsZero())
 	assert.Equal(t, hit.internalDontTouch.deltaScreenLines, 0)
 }
 
 func TestFindFirstHitAnsi(t *testing.T) {
-	reader := NewReaderFromStream("", strings.NewReader("A\x1b[30mB"))
+	reader := NewReaderFromText("", "A\x1b[30mB")
 	pager := NewPager(reader)
 	pager.screen = twin.NewFakeScreen(40, 10)
 
@@ -289,12 +295,12 @@ func TestFindFirstHitAnsi(t *testing.T) {
 	pager.searchPattern = toPattern("AB")
 
 	hit := pager.findFirstHit(newScrollPosition("TestFindFirstHitSimple"), false)
-	assert.Equal(t, hit.internalDontTouch.lineNumberOneBased, 1)
+	assert.Assert(t, hit.internalDontTouch.lineNumber.IsZero())
 	assert.Equal(t, hit.internalDontTouch.deltaScreenLines, 0)
 }
 
 func TestFindFirstHitNoMatch(t *testing.T) {
-	reader := NewReaderFromStream("TestFindFirstHitSimple", strings.NewReader("AB"))
+	reader := NewReaderFromText("TestFindFirstHitSimple", "AB")
 	pager := NewPager(reader)
 	pager.screen = twin.NewFakeScreen(40, 10)
 
@@ -319,8 +325,8 @@ func rowToString(row []twin.Cell) string {
 }
 
 func TestScrollToBottomWrapNextToLastLine(t *testing.T) {
-	reader := NewReaderFromStream("",
-		strings.NewReader("first line\nline two will be wrapped\nhere's the last line"))
+	reader := NewReaderFromText("",
+		"first line\nline two will be wrapped\nhere's the last line")
 
 	// Heigh 3 = two lines of contents + one footer
 	screen := twin.NewFakeScreen(10, 3)
@@ -456,7 +462,7 @@ func TestIsScrolledToEnd_WrappedLastLine(t *testing.T) {
 	pager.scrollToEnd()
 	assert.Equal(t, true, pager.isScrolledToEnd())
 
-	pager.onKey(twin.KeyUp)
+	pager.mode.onKey(twin.KeyUp)
 	pager.redraw("XXX")
 	assert.Equal(t, false, pager.isScrolledToEnd())
 }
@@ -481,41 +487,51 @@ func TestIsScrolledToEnd_EmptyFile(t *testing.T) {
 // Verify that we can page all files in ../sample-files/* without crashing
 func TestPageSamples(t *testing.T) {
 	for _, fileName := range getTestFiles() {
-		file, err := os.Open(fileName)
-		if err != nil {
-			t.Errorf("Error opening file <%s>: %s", fileName, err.Error())
-			continue
-		}
-		defer func() {
-			if err := file.Close(); err != nil {
-				panic(err)
+		t.Run(fileName, func(t *testing.T) {
+			file, err := os.Open(fileName)
+			if err != nil {
+				t.Errorf("Error opening file <%s>: %s", fileName, err.Error())
+				return
 			}
-		}()
+			defer func() {
+				if err := file.Close(); err != nil {
+					panic(err)
+				}
+			}()
 
-		myReader := NewReaderFromStream(fileName, file)
-		for !myReader.done.Load() {
-		}
+			myReader := NewReaderFromStream(fileName, file, chroma.Style{}, nil, nil)
+			for !myReader.done.Load() {
+			}
 
-		pager := NewPager(myReader)
-		pager.WrapLongLines = false
-		pager.ShowLineNumbers = false
+			pager := NewPager(myReader)
+			pager.WrapLongLines = false
+			pager.ShowLineNumbers = false
 
-		// Heigh 3 = two lines of contents + one footer
-		screen := twin.NewFakeScreen(10, 3)
+			// Heigh 3 = two lines of contents + one footer
+			screen := twin.NewFakeScreen(10, 3)
 
-		// Exit immediately
-		pager.Quit()
+			// Exit immediately
+			pager.Quit()
 
-		// Get contents onto our fake screen
-		pager.StartPaging(screen, nil, nil)
-		pager.redraw("")
+			// Get contents onto our fake screen
+			pager.StartPaging(screen, nil, nil)
+			pager.redraw("")
 
-		firstReaderLine := myReader.GetLine(0)
-		if firstReaderLine == nil {
-			continue
-		}
-		firstPagerLine := rowToString(screen.GetRow(0))
-		assert.Assert(t, strings.HasPrefix(firstReaderLine.Plain(nil), firstPagerLine))
+			firstReaderLine := myReader.GetLine(linenumbers.LineNumber{})
+			if firstReaderLine == nil {
+				return
+			}
+			firstPagerLine := rowToString(screen.GetRow(0))
+
+			// Handle the case when first line is chopped off to the right
+			firstPagerLine = strings.TrimSuffix(firstPagerLine, ">")
+
+			assert.Assert(t,
+				strings.HasPrefix(firstReaderLine.Plain(nil), firstPagerLine),
+				"\nreader line = <%s>\npager line  = <%s>",
+				firstReaderLine.Plain(nil), firstPagerLine,
+			)
+		})
 	}
 }
 
@@ -605,10 +621,15 @@ func benchmarkSearch(b *testing.B, highlighted bool) {
 		panic("Getting current filename failed")
 	}
 
+	sourceBytes, err := os.ReadFile(sourceFilename)
+	if err != nil {
+		panic(err)
+	}
+	fileContents := string(sourceBytes)
+
 	// Read one copy of the example input
-	var fileContents string
 	if highlighted {
-		highlightedSourceCode, err := highlight(sourceFilename, true, *styles.Get("native"), formatters.TTY16m)
+		highlightedSourceCode, err := highlight(fileContents, *styles.Get("native"), formatters.TTY16m, lexers.Get("go"))
 		if err != nil {
 			panic(err)
 		}
@@ -616,12 +637,6 @@ func benchmarkSearch(b *testing.B, highlighted bool) {
 			panic("Highlighting didn't want to, returned nil")
 		}
 		fileContents = *highlightedSourceCode
-	} else {
-		sourceBytes, err := os.ReadFile(sourceFilename)
-		if err != nil {
-			panic(err)
-		}
-		fileContents = string(sourceBytes)
 	}
 
 	// Duplicate data N times
@@ -638,14 +653,6 @@ func benchmarkSearch(b *testing.B, highlighted bool) {
 	// we're searching through this very file.
 	pager.searchPattern = regexp.MustCompile("This won'[t] match anything")
 
-	// Wait for reader to finish reading...
-	for !reader.done.Load() {
-	}
-
-	// ... and wait for highlighting to finish
-	for !reader.highlightingDone.Load() {
-	}
-
 	// I hope forcing a GC here will make numbers more predictable
 	runtime.GC()
 
