@@ -5,19 +5,25 @@ import (
 	"bytes"
 	"fmt"
 	"io"
-	"math"
 	"os"
-	"os/exec"
 	"path"
 	"strings"
 	"sync"
 	"sync/atomic"
 	"time"
 
+	"github.com/walles/moar/m/linenumbers"
+
 	"github.com/alecthomas/chroma/v2"
+	"github.com/alecthomas/chroma/v2/lexers"
 	log "github.com/sirupsen/logrus"
 )
 
+// Files larger than this won't be highlighted
+//
+//revive:disable-next-line:var-naming
+const MAX_HIGHLIGHT_SIZE int64 = 1024 * 1024
+
 // Reader reads a file into an array of strings.
 //
 // It does the reading in the background, and it returns parts of the read data
@@ -27,10 +33,9 @@ import (
 type Reader struct {
 	sync.Mutex
 
-	lines   []*Line
-	name    *string
-	err     error
-	_stderr io.Reader
+	lines []*Line
+	name  *string
+	err   error
 
 	// Have we had our contents replaced using setText()?
 	replaced bool
@@ -50,68 +55,12 @@ type InputLines struct {
 	lines []*Line
 
 	// One-based line number of the first line returned
-	firstLineOneBased int
+	firstLine linenumbers.LineNumber
 
 	// "monkey.txt: 1-23/45 51%"
 	statusText string
 }
 
-// Shut down the filter (if any) after we're done reading the file.
-func (reader *Reader) cleanupFilter(fromFilter *exec.Cmd) {
-	defer func() {
-		reader.done.Store(true)
-		select {
-		case reader.maybeDone <- true:
-		default:
-		}
-	}()
-
-	// FIXME: Close the stream now that we're done reading it?
-
-	if fromFilter == nil {
-		log.Trace("Reader done, no filter")
-		return
-	}
-
-	reader.Lock()
-	defer reader.Unlock()
-
-	// Give the filter a little time to go away
-	timer := time.AfterFunc(2*time.Second, func() {
-		// FIXME: Regarding error handling, maybe we should log all errors
-		// except for "process doesn't exist"? If the process is not there
-		// it likely means the process finished up by itself without our
-		// help.
-		_ = fromFilter.Process.Kill()
-	})
-
-	stderrText := ""
-	if reader._stderr != nil {
-		// Drain the reader's stderr into a string for possible inclusion in an error message
-		// From: https://stackoverflow.com/a/9650373/473672
-		if buffer, err := io.ReadAll(reader._stderr); err == nil {
-			stderrText = strings.TrimSpace(string(buffer))
-		} else {
-			log.Warn("Draining filter stderr failed: ", err)
-		}
-	}
-
-	err := fromFilter.Wait()
-	timer.Stop()
-
-	// Don't overwrite any existing problem report
-	if reader.err == nil {
-		reader.err = err
-		if err != nil && stderrText != "" {
-			reader.err = fmt.Errorf("%s: %w", stderrText, err)
-		}
-	}
-
-	// FIXME: Report any filter printouts to stderr to the user
-
-	log.Trace("Reader done, filter done")
-}
-
 // Count lines in the original file and preallocate space for them.  Good
 // performance improvement:
 //
@@ -146,8 +95,14 @@ func (reader *Reader) preAllocLines(originalFileName string) {
 }
 
 // This function will be update the Reader struct in the background.
-func (reader *Reader) readStream(stream io.Reader, originalFileName *string, fromFilter *exec.Cmd) {
-	defer reader.cleanupFilter(fromFilter)
+func (reader *Reader) readStream(stream io.Reader, originalFileName *string, onDone func()) {
+	defer func() {
+		reader.done.Store(true)
+		select {
+		case reader.maybeDone <- true:
+		default:
+		}
+	}()
 
 	if originalFileName != nil {
 		reader.preAllocLines(*originalFileName)
@@ -212,6 +167,10 @@ func (reader *Reader) readStream(stream io.Reader, originalFileName *string, fro
 		}
 	}
 
+	if onDone != nil {
+		onDone()
+	}
+
 	t1 := time.Now().UnixNano()
 	dtNanos := t1 - t0
 	log.Debug("Stream read in ", dtNanos/1_000_000, "ms")
@@ -223,9 +182,8 @@ func (reader *Reader) readStream(stream io.Reader, originalFileName *string, fro
 //
 // If non-empty, the name will be displayed by the pager in the bottom left
 // corner to help the user keep track of what is being paged.
-func NewReaderFromStream(name string, reader io.Reader) *Reader {
-	mReader := newReaderFromStream(reader, nil, nil)
-	mReader.highlightingDone.Store(true) // No highlighting of streams = nothing left to do = Done!
+func NewReaderFromStream(name string, reader io.Reader, style chroma.Style, formatter chroma.Formatter, lexer chroma.Lexer) *Reader {
+	mReader := newReaderFromStream(reader, nil, style, formatter, lexer)
 
 	if len(name) > 0 {
 		mReader.Lock()
@@ -245,7 +203,9 @@ func NewReaderFromStream(name string, reader io.Reader) *Reader {
 //
 // If fromFilter is not nil this method will wait() for it, and effectively
 // takes over ownership for it.
-func newReaderFromStream(reader io.Reader, originalFileName *string, fromFilter *exec.Cmd) *Reader {
+//
+// If lexer is not nil, the file will be highlighted after being fully read.
+func newReaderFromStream(reader io.Reader, originalFileName *string, style chroma.Style, formatter chroma.Formatter, lexer chroma.Lexer) *Reader {
 	done := atomic.Bool{}
 	done.Store(false)
 	highlightingDone := atomic.Bool{}
@@ -262,7 +222,19 @@ func newReaderFromStream(reader io.Reader, originalFileName *string, fromFilter
 
 	// FIXME: Make sure that if we panic somewhere inside of this goroutine,
 	// the main program terminates and prints our panic stack trace.
-	go returnMe.readStream(reader, originalFileName, fromFilter)
+	go returnMe.readStream(reader, originalFileName, func() {
+		if lexer == nil {
+			return
+		}
+
+		highlightFromMemory(&returnMe, style, formatter, lexer)
+
+		returnMe.highlightingDone.Store(true)
+		select {
+		case returnMe.maybeDone <- true:
+		default:
+		}
+	})
 
 	return &returnMe
 }
@@ -288,46 +260,17 @@ func NewReaderFromText(name string, text string) *Reader {
 	highlightingDone := atomic.Bool{}
 	highlightingDone.Store(true) // No highlighting to do = nothing left = Done!
 	returnMe := &Reader{
-		name:             &name,
 		lines:            lines,
 		done:             &done,
 		highlightingDone: &highlightingDone,
 	}
+	if name != "" {
+		returnMe.name = &name
+	}
 
 	return returnMe
 }
 
-// newReaderFromCommand creates a new reader by running a file through a filter
-func newReaderFromCommand(filename string, filterCommand ...string) (*Reader, error) {
-	filterWithFilename := append(filterCommand, filename)
-	filter := exec.Command(filterWithFilename[0], filterWithFilename[1:]...)
-
-	filterOut, err := filter.StdoutPipe()
-	if err != nil {
-		return nil, err
-	}
-
-	filterErr, err := filter.StderrPipe()
-	if err != nil {
-		// The error stream is only used in case of failures, and having it
-		// nil is fine, so just log this and move along.
-		log.Warnf("Stderr not available from %s: %s", filterCommand[0], err.Error())
-	}
-
-	err = filter.Start()
-	if err != nil {
-		return nil, err
-	}
-
-	reader := newReaderFromStream(filterOut, nil, filter)
-	reader.highlightingDone.Store(true) // No highlighting to do == nothing left == Done!
-	reader.Lock()
-	reader.name = &filename
-	reader._stderr = filterErr
-	reader.Unlock()
-	return reader, nil
-}
-
 // Duplicate of moar/moar.go:tryOpen
 func tryOpen(filename string) error {
 	// Try opening the file
@@ -359,7 +302,7 @@ func countLines(filename string) (uint64, error) {
 	const lineBreak = '\n'
 	sliceWithSingleLineBreak := []byte{lineBreak}
 
-	reader, err := os.Open(filename)
+	reader, err := ZOpen(filename)
 	if err != nil {
 		return 0, err
 	}
@@ -407,47 +350,74 @@ func countLines(filename string) (uint64, error) {
 
 // NewReaderFromFilename creates a new file reader.
 //
+// If lexer is nil it will be determined from the input file name.
+//
 // The Reader will try to uncompress various compressed file format, and also
 // apply highlighting to the file using Chroma:
 // https://github.com/alecthomas/chroma
-func NewReaderFromFilename(filename string, style chroma.Style, formatter chroma.Formatter) (*Reader, error) {
+func NewReaderFromFilename(filename string, style chroma.Style, formatter chroma.Formatter, lexer chroma.Lexer) (*Reader, error) {
 	fileError := tryOpen(filename)
 	if fileError != nil {
 		return nil, fileError
 	}
 
-	if strings.HasSuffix(filename, ".gz") {
-		return newReaderFromCommand(filename, "gzip", "-d", "-c")
-	}
-	if strings.HasSuffix(filename, ".bz2") {
-		return newReaderFromCommand(filename, "bzip2", "-d", "-c")
-	}
-	if strings.HasSuffix(filename, ".xz") {
-		return newReaderFromCommand(filename, "xz", "-d", "-c")
-	}
-
-	stream, err := os.Open(filename)
+	stream, err := ZOpen(filename)
 	if err != nil {
 		return nil, err
 	}
 
-	returnMe := newReaderFromStream(stream, &filename, nil)
+	// Set lexer to nil in this call since we want to do our own highlighting in
+	// parallel with the stream being read. See the call to
+	// StartHighlightingFromFile() below.
+	returnMe := newReaderFromStream(stream, &filename, chroma.Style{}, nil, nil)
+
 	returnMe.Lock()
 	returnMe.name = &filename
 	returnMe.Unlock()
 
+	startHighlightingFromFile(returnMe, filename, style, formatter, lexer)
+
+	return returnMe, nil
+}
+
+func startHighlightingFromFile(reader *Reader, filename string, style chroma.Style, formatter chroma.Formatter, lexer chroma.Lexer) {
+	reportDone := func() {
+		reader.highlightingDone.Store(true)
+		select {
+		case reader.maybeDone <- true:
+		default:
+		}
+
+		log.Trace("Highlighting done")
+	}
+
+	fileInfo, err := os.Stat(filename)
+	if err != nil {
+		log.Warn("Failed to stat file for highlighting: ", err)
+		reportDone()
+		return
+	}
+	if fileInfo.Size() > MAX_HIGHLIGHT_SIZE {
+		log.Debug("File too large for highlighting: ", fileInfo.Size())
+		reportDone()
+		return
+	}
+
 	go func() {
-		defer func() {
-			returnMe.highlightingDone.Store(true)
-			select {
-			case returnMe.maybeDone <- true:
-			default:
-			}
+		defer reportDone()
 
-			log.Trace("Highlighting done")
-		}()
+		fileBytes, err := os.ReadFile(filename)
+		if err != nil {
+			log.Warn("Failed to read file for highlighting: ", err)
+			return
+		}
 
-		highlighted, err := highlight(filename, false, style, formatter)
+		if lexer == nil {
+			// Try auto detecting by filename
+			lexer = lexers.Match(filename)
+		}
+
+		highlighted, err := highlight(string(fileBytes), style, formatter, lexer)
 		if err != nil {
 			log.Warn("Highlighting failed: ", err)
 			return
@@ -458,122 +428,145 @@ func NewReaderFromFilename(filename string, style chroma.Style, formatter chroma
 			return
 		}
 
-		returnMe.setText(*highlighted)
+		reader.setText(*highlighted)
+	}()
+}
+
+func highlightFromMemory(reader *Reader, style chroma.Style, formatter chroma.Formatter, lexer chroma.Lexer) {
+	defer func() {
+		reader.highlightingDone.Store(true)
+		select {
+		case reader.maybeDone <- true:
+		default:
+		}
 	}()
 
-	return returnMe, nil
+	if lexer == nil {
+		return
+	}
+
+	var byteCount int64
+	reader.Lock()
+	for _, line := range reader.lines {
+		byteCount += int64(len(line.raw))
+	}
+	reader.Unlock()
+
+	if byteCount > MAX_HIGHLIGHT_SIZE {
+		log.Debug("File too large for highlighting: ", byteCount)
+		return
+	}
+
+	textBuilder := strings.Builder{}
+	reader.Lock()
+	for _, line := range reader.lines {
+		textBuilder.WriteString(line.raw)
+		textBuilder.WriteString("\n")
+	}
+	reader.Unlock()
+
+	highlighted, err := highlight(textBuilder.String(), style, formatter, lexer)
+	if err != nil {
+		log.Warn("Highlighting failed: ", err)
+		return
+	}
+
+	if highlighted == nil {
+		// No highlighting would be done, never mind
+		return
+	}
+
+	reader.setText(*highlighted)
 }
 
 // createStatusUnlocked() assumes that its caller is holding the lock
-func (r *Reader) createStatusUnlocked(lastLineOneBased int) string {
+func (reader *Reader) createStatusUnlocked(lastLine linenumbers.LineNumber) string {
 	prefix := ""
-	if r.name != nil {
-		prefix = path.Base(*r.name) + ": "
+	if reader.name != nil {
+		prefix = path.Base(*reader.name) + ": "
 	}
 
-	if len(r.lines) == 0 {
+	if len(reader.lines) == 0 {
 		return prefix + "<empty>"
 	}
 
-	if len(r.lines) == 1 {
+	if len(reader.lines) == 1 {
 		return prefix + "1 line  100%"
 	}
 
-	percent := int(100 * float64(lastLineOneBased) / float64(len(r.lines)))
+	percent := int(100 * float64(lastLine.AsOneBased()) / float64(len(reader.lines)))
 
 	return fmt.Sprintf("%s%s lines  %d%%",
 		prefix,
-		formatNumber(uint(len(r.lines))),
+		linenumbers.LineNumberFromLength(len(reader.lines)).Format(),
 		percent)
 }
 
 // GetLineCount returns the number of lines available for viewing
-func (r *Reader) GetLineCount() int {
-	r.Lock()
-	defer r.Unlock()
+func (reader *Reader) GetLineCount() int {
+	reader.Lock()
+	defer reader.Unlock()
 
-	return len(r.lines)
+	return len(reader.lines)
 }
 
 // GetLine gets a line. If the requested line number is out of bounds, nil is returned.
-func (r *Reader) GetLine(lineNumberOneBased int) *Line {
-	r.Lock()
-	defer r.Unlock()
+func (reader *Reader) GetLine(lineNumber linenumbers.LineNumber) *Line {
+	reader.Lock()
+	defer reader.Unlock()
 
-	if lineNumberOneBased < 1 {
+	if lineNumber.AsOneBased() > len(reader.lines) {
 		return nil
 	}
-	if lineNumberOneBased > len(r.lines) {
-		return nil
-	}
-	return r.lines[lineNumberOneBased-1]
+	return reader.lines[lineNumber.AsZeroBased()]
 }
 
 // GetLines gets the indicated lines from the input
 //
 // Overflow state will be didFit if we returned all lines we currently have, or
 // didOverflow otherwise.
-func (r *Reader) GetLines(firstLineOneBased int, wantedLineCount int) (*InputLines, overflowState) {
-	r.Lock()
-	defer r.Unlock()
-	return r.getLinesUnlocked(firstLineOneBased, wantedLineCount)
+//
+//revive:disable-next-line:unexported-return
+func (reader *Reader) GetLines(firstLine linenumbers.LineNumber, wantedLineCount int) (*InputLines, overflowState) {
+	reader.Lock()
+	defer reader.Unlock()
+	return reader.getLinesUnlocked(firstLine, wantedLineCount)
 }
 
-func nonWrappingAdd(a int, b int) int {
-	if b < 0 {
-		panic(fmt.Sprintf("b must be >= 0, was: %d", b))
-	}
-
-	if a > math.MaxInt-b {
-		return math.MaxInt
-	}
-
-	return a + b
-}
-
-func (r *Reader) getLinesUnlocked(firstLineOneBased int, wantedLineCount int) (*InputLines, overflowState) {
-	if firstLineOneBased < 1 {
-		firstLineOneBased = 1
-	}
-
-	if len(r.lines) == 0 || wantedLineCount == 0 {
+func (reader *Reader) getLinesUnlocked(firstLine linenumbers.LineNumber, wantedLineCount int) (*InputLines, overflowState) {
+	if len(reader.lines) == 0 || wantedLineCount == 0 {
 		return &InputLines{
-				lines:             nil,
-				firstLineOneBased: firstLineOneBased,
-				statusText:        r.createStatusUnlocked(firstLineOneBased),
+				lines:      nil,
+				firstLine:  firstLine,
+				statusText: reader.createStatusUnlocked(firstLine),
 			},
 			didFit // Empty files always fit
 	}
 
-	firstLineZeroBased := firstLineOneBased - 1
-	lastLineZeroBased := nonWrappingAdd(firstLineZeroBased, wantedLineCount-1)
-
-	if lastLineZeroBased >= len(r.lines) {
-		lastLineZeroBased = len(r.lines) - 1
-	}
+	lastLine := firstLine.NonWrappingAdd(wantedLineCount - 1)
 
 	// Prevent reading past the end of the available lines
-	actualLineCount := lastLineZeroBased - firstLineZeroBased + 1
-	if actualLineCount < wantedLineCount && firstLineOneBased > 1 {
-		overshoot := wantedLineCount - actualLineCount
-		firstLineOneBased -= overshoot
-		if firstLineOneBased < 1 {
-			firstLineOneBased = 1
-		}
+	maxLineNumber := *linenumbers.LineNumberFromLength(len(reader.lines))
+	if lastLine.IsAfter(maxLineNumber) {
+		lastLine = maxLineNumber
 
-		return r.getLinesUnlocked(firstLineOneBased, wantedLineCount)
+		// If one line was requested, then first and last should be exactly the
+		// same, and we would get there by adding zero.
+		firstLine = lastLine.NonWrappingAdd(1 - wantedLineCount)
+
+		return reader.getLinesUnlocked(firstLine, firstLine.CountLinesTo(lastLine))
 	}
 
-	returnLines := r.lines[firstLineZeroBased : lastLineZeroBased+1]
+	returnLines := reader.lines[firstLine.AsZeroBased() : lastLine.AsZeroBased()+1]
 	overflow := didFit
-	if len(returnLines) != len(r.lines) {
+	if len(returnLines) != len(reader.lines) {
 		overflow = didOverflow // We're not returning all available lines
 	}
 
 	return &InputLines{
-			lines:             returnLines,
-			firstLineOneBased: firstLineOneBased,
-			statusText:        r.createStatusUnlocked(lastLineZeroBased + 1),
+			lines:      returnLines,
+			firstLine:  firstLine,
+			statusText: reader.createStatusUnlocked(lastLine),
 		},
 		overflow
 }
