@@ -1,20 +1,23 @@
 package m
 
 import (
-	"math"
 	"os"
 	"os/exec"
 	"path"
-	"runtime"
 	"strconv"
 	"strings"
 	"testing"
 
 	"github.com/alecthomas/chroma/v2/formatters"
 	"github.com/alecthomas/chroma/v2/styles"
+	"github.com/walles/moar/m/linenumbers"
 	"gotest.tools/v3/assert"
 )
 
+//revive:disable:empty-block
+
+const samplesDir = "../sample-files"
+
 func testGetLineCount(t *testing.T, reader *Reader) {
 	if strings.Contains(*reader.name, "compressed") {
 		// We are no good at counting lines of compressed files, never mind
@@ -56,7 +59,7 @@ func testGetLineCount(t *testing.T, reader *Reader) {
 }
 
 func testGetLines(t *testing.T, reader *Reader) {
-	lines, _ := reader.GetLines(1, 10)
+	lines, _ := reader.GetLines(linenumbers.LineNumber{}, 10)
 	if len(lines.lines) > 10 {
 		t.Errorf("Asked for 10 lines, got too many: %d", len(lines.lines))
 	}
@@ -68,17 +71,17 @@ func testGetLines(t *testing.T, reader *Reader) {
 	}
 
 	// Test clipping at the end
-	lines, _ = reader.GetLines(math.MaxInt32, 10)
+	lines, _ = reader.GetLines(linenumbers.LineNumberMax(), 10)
 	if len(lines.lines) != 10 {
 		t.Errorf("Asked for 10 lines but got %d", len(lines.lines))
 		return
 	}
 
-	startOfLastSection := lines.firstLineOneBased
+	startOfLastSection := lines.firstLine
 	lines, _ = reader.GetLines(startOfLastSection, 10)
-	if lines.firstLineOneBased != startOfLastSection {
-		t.Errorf("Expected start line %d when asking for the last 10 lines, got %d",
-			startOfLastSection, lines.firstLineOneBased)
+	if lines.firstLine != startOfLastSection {
+		t.Errorf("Expected start line %d when asking for the last 10 lines, got %s",
+			startOfLastSection, lines.firstLine.Format())
 		return
 	}
 	if len(lines.lines) != 10 {
@@ -87,10 +90,10 @@ func testGetLines(t *testing.T, reader *Reader) {
 		return
 	}
 
-	lines, _ = reader.GetLines(startOfLastSection+1, 10)
-	if lines.firstLineOneBased != startOfLastSection {
-		t.Errorf("Expected start line %d when asking for the last+1 10 lines, got %d",
-			startOfLastSection, lines.firstLineOneBased)
+	lines, _ = reader.GetLines(startOfLastSection.NonWrappingAdd(1), 10)
+	if lines.firstLine != startOfLastSection {
+		t.Errorf("Expected start line %d when asking for the last+1 10 lines, got %s",
+			startOfLastSection, lines.firstLine.Format())
 		return
 	}
 	if len(lines.lines) != 10 {
@@ -99,10 +102,10 @@ func testGetLines(t *testing.T, reader *Reader) {
 		return
 	}
 
-	lines, _ = reader.GetLines(startOfLastSection-1, 10)
-	if lines.firstLineOneBased != startOfLastSection-1 {
-		t.Errorf("Expected start line %d when asking for the last-1 10 lines, got %d",
-			startOfLastSection, lines.firstLineOneBased)
+	lines, _ = reader.GetLines(startOfLastSection.NonWrappingAdd(-1), 10)
+	if lines.firstLine != startOfLastSection.NonWrappingAdd(-1) {
+		t.Errorf("Expected start line %d when asking for the last-1 10 lines, got %s",
+			startOfLastSection, lines.firstLine.Format())
 		return
 	}
 	if len(lines.lines) != 10 {
@@ -112,25 +115,15 @@ func testGetLines(t *testing.T, reader *Reader) {
 	}
 }
 
-func getSamplesDir() string {
-	// From: https://coderwall.com/p/_fmbug/go-get-path-to-current-file
-	_, filename, _, ok := runtime.Caller(0)
-	if !ok {
-		panic("Getting current filename failed")
-	}
-
-	return path.Join(path.Dir(filename), "../sample-files")
-}
-
 func getTestFiles() []string {
-	files, err := os.ReadDir(getSamplesDir())
+	files, err := os.ReadDir(samplesDir)
 	if err != nil {
 		panic(err)
 	}
 
 	var filenames []string
 	for _, file := range files {
-		filenames = append(filenames, "../sample-files/"+file.Name())
+		filenames = append(filenames, path.Join(samplesDir, file.Name()))
 	}
 
 	return filenames
@@ -139,8 +132,10 @@ func getTestFiles() []string {
 // Wait for reader to finish reading and highlighting. Used by tests.
 func (r *Reader) _wait() error {
 	// Wait for our goroutine to finish
+	//revive:disable-next-line:empty-block
 	for !r.done.Load() {
 	}
+	//revive:disable-next-line:empty-block
 	for !r.highlightingDone.Load() {
 	}
 
@@ -167,7 +162,7 @@ func TestGetLines(t *testing.T) {
 			}
 		}
 
-		reader, err := NewReaderFromFilename(file, *styles.Get("native"), formatters.TTY16m)
+		reader, err := NewReaderFromFilename(file, *styles.Get("native"), formatters.TTY16m, nil)
 		if err != nil {
 			t.Errorf("Error opening file <%s>: %s", file, err.Error())
 			continue
@@ -196,6 +191,12 @@ func testHighlightingLineCount(t *testing.T, filenameWithPath string) {
 	if strings.HasSuffix(filenameWithPath, ".gz") {
 		return
 	}
+	if strings.HasSuffix(filenameWithPath, ".zst") {
+		return
+	}
+	if strings.HasSuffix(filenameWithPath, ".zstd") {
+		return
+	}
 
 	// Load the unformatted file
 	rawBytes, err := os.ReadFile(filenameWithPath)
@@ -213,11 +214,11 @@ func testHighlightingLineCount(t *testing.T, filenameWithPath string) {
 	}
 	rawLinesCount := rawLinefeedsCount
 	if !rawFileEndsWithNewline {
-		rawLinesCount += 1
+		rawLinesCount++
 	}
 
 	// Then load the same file using one of our Readers
-	reader, err := NewReaderFromFilename(filenameWithPath, *styles.Get("native"), formatters.TTY16m)
+	reader, err := NewReaderFromFilename(filenameWithPath, *styles.Get("native"), formatters.TTY16m, nil)
 	if err != nil {
 		panic(err)
 	}
@@ -227,12 +228,12 @@ func testHighlightingLineCount(t *testing.T, filenameWithPath string) {
 	}
 
 	highlightedLinesCount := reader.GetLineCount()
-	assert.Check(t, rawLinesCount == highlightedLinesCount)
+	assert.Equal(t, rawLinesCount, highlightedLinesCount)
 }
 
 func TestGetLongLine(t *testing.T) {
 	file := "../sample-files/very-long-line.txt"
-	reader, err := NewReaderFromFilename(file, *styles.Get("native"), formatters.TTY16m)
+	reader, err := NewReaderFromFilename(file, *styles.Get("native"), formatters.TTY16m, nil)
 	if err != nil {
 		panic(err)
 	}
@@ -240,8 +241,8 @@ func TestGetLongLine(t *testing.T) {
 		panic(err)
 	}
 
-	lines, overflow := reader.GetLines(1, 5)
-	assert.Equal(t, lines.firstLineOneBased, 1)
+	lines, overflow := reader.GetLines(linenumbers.LineNumber{}, 5)
+	assert.Equal(t, lines.firstLine, linenumbers.LineNumber{})
 	assert.Equal(t, len(lines.lines), 1)
 
 	// This fits because we got all (one) input lines. Given the line length the
@@ -257,32 +258,31 @@ func TestGetLongLine(t *testing.T) {
 }
 
 func getReaderWithLineCount(totalLines int) *Reader {
-	reader := NewReaderFromStream("", strings.NewReader(strings.Repeat("x\n", totalLines)))
-	if err := reader._wait(); err != nil {
-		panic(err)
-	}
-
-	return reader
+	return NewReaderFromText("", strings.Repeat("x\n", totalLines))
 }
 
-func testStatusText(t *testing.T, fromLine int, toLine int, totalLines int, expected string) {
+func testStatusText(t *testing.T, fromLine linenumbers.LineNumber, toLine linenumbers.LineNumber, totalLines int, expected string) {
 	testMe := getReaderWithLineCount(totalLines)
-	linesRequested := toLine - fromLine + 1
+	linesRequested := fromLine.CountLinesTo(toLine)
 	lines, _ := testMe.GetLines(fromLine, linesRequested)
 	statusText := lines.statusText
 	assert.Equal(t, statusText, expected)
 }
 
 func TestStatusText(t *testing.T) {
-	testStatusText(t, 1, 10, 20, "20 lines  50%")
-	testStatusText(t, 1, 5, 5, "5 lines  100%")
-	testStatusText(t, 998, 999, 1000, "1000 lines  99%")
+	testStatusText(t, linenumbers.LineNumber{}, linenumbers.LineNumberFromOneBased(10), 20, "20 lines  50%")
+	testStatusText(t, linenumbers.LineNumber{}, linenumbers.LineNumberFromOneBased(5), 5, "5 lines  100%")
+	testStatusText(t,
+		linenumbers.LineNumberFromOneBased(998),
+		linenumbers.LineNumberFromOneBased(999),
+		1000,
+		"1000 lines  99%")
 
-	testStatusText(t, 0, 0, 0, "<empty>")
-	testStatusText(t, 1, 1, 1, "1 line  100%")
+	testStatusText(t, linenumbers.LineNumber{}, linenumbers.LineNumber{}, 0, "<empty>")
+	testStatusText(t, linenumbers.LineNumber{}, linenumbers.LineNumber{}, 1, "1 line  100%")
 
 	// Test with filename
-	testMe, err := NewReaderFromFilename(getSamplesDir()+"/empty", *styles.Get("native"), formatters.TTY16m)
+	testMe, err := NewReaderFromFilename(samplesDir+"/empty", *styles.Get("native"), formatters.TTY16m, nil)
 	if err != nil {
 		panic(err)
 	}
@@ -290,14 +290,17 @@ func TestStatusText(t *testing.T) {
 		panic(err)
 	}
 
-	line, overflow := testMe.GetLines(0, 0)
+	line, overflow := testMe.GetLines(linenumbers.LineNumber{}, 0)
+	if line.lines != nil {
+		t.Error("line.lines is should have been nil when reading from an empty stream")
+	}
 	assert.Equal(t, line.statusText, "empty: <empty>")
 	assert.Equal(t, overflow, didFit) // Empty always fits
 }
 
 func testCompressedFile(t *testing.T, filename string) {
-	filenameWithPath := getSamplesDir() + "/" + filename
-	reader, e := NewReaderFromFilename(filenameWithPath, *styles.Get("native"), formatters.TTY16m)
+	filenameWithPath := path.Join(samplesDir, filename)
+	reader, e := NewReaderFromFilename(filenameWithPath, *styles.Get("native"), formatters.TTY16m, nil)
 	if e != nil {
 		t.Errorf("Error opening file <%s>: %s", filenameWithPath, e.Error())
 		panic(e)
@@ -306,58 +309,32 @@ func testCompressedFile(t *testing.T, filename string) {
 		panic(err)
 	}
 
-	lines, _ := reader.GetLines(1, 5)
+	lines, _ := reader.GetLines(linenumbers.LineNumber{}, 5)
 	assert.Equal(t, lines.lines[0].Plain(nil), "This is a compressed file", "%s", filename)
 }
 
 func TestCompressedFiles(t *testing.T) {
 	testCompressedFile(t, "compressed.txt.gz")
-
-	_, err := exec.LookPath("bzip2")
-	if err == nil {
-		testCompressedFile(t, "compressed.txt.bz2")
-	} else {
-		t.Log("WARNING: bzip2 not found in path, not testing automatic bzip2 decompression")
-	}
-
-	_, err = exec.LookPath("xz")
-	if err == nil {
-		testCompressedFile(t, "compressed.txt.xz")
-	} else {
-		t.Log("WARNING: xz not found in path, not testing automatic xz decompression")
-	}
+	testCompressedFile(t, "compressed.txt.bz2")
+	testCompressedFile(t, "compressed.txt.xz")
+	testCompressedFile(t, "compressed.txt.zst")
+	testCompressedFile(t, "compressed.txt.zstd")
 }
 
 func TestFilterNotInstalled(t *testing.T) {
-	// FIXME: Test what happens if we try to use a filter that is not installed
+	t.Skip("FIXME: Test what happens if we try to use a filter that is not installed")
 }
 
 func TestFilterFailure(t *testing.T) {
-	// FIXME: Test what happens if the filter command fails because of bad command line options
+	t.Skip("FIXME: Test what happens if the filter command fails because of bad command line options")
 }
 
 func TestFilterPermissionDenied(t *testing.T) {
-	// FIXME: Test what happens if the filter command fails because it can't access the requested file
-}
-
-func TestFilterFileNotFound(t *testing.T) {
-	// What happens if the filter cannot read its input file?
-	NonExistentPath := "/does-not-exist"
-
-	reader, err := newReaderFromCommand(NonExistentPath, "cat")
-
-	// Creating should be fine, it's waiting for it to finish that should fail.
-	// Feel free to re-evaluate in the future.
-	assert.Check(t, err == nil)
-
-	err = reader._wait()
-	assert.Check(t, err != nil)
-
-	assert.Check(t, strings.Contains(err.Error(), NonExistentPath), err.Error())
+	t.Skip("FIXME: Test what happens if the filter command fails because it can't access the requested file")
 }
 
 func TestFilterNotAFile(t *testing.T) {
-	// FIXME: Test what happens if the filter command fails because the target is not a file
+	t.Skip("FIXME: Test what happens if the filter command fails because the target is not a file")
 }
 
 // How long does it take to read a file?
@@ -366,16 +343,17 @@ func TestFilterNotAFile(t *testing.T) {
 //
 // Run with: go test -run='^$' -bench=. . ./...
 func BenchmarkReaderDone(b *testing.B) {
-	filename := getSamplesDir() + "/../m/pager.go" // This is our longest .go file
+	filename := "pager.go" // This is our longest .go file
 	b.ResetTimer()
 	for n := 0; n < b.N; n++ {
 		// This is our longest .go file
-		readMe, err := NewReaderFromFilename(filename, *styles.Get("native"), formatters.TTY16m)
+		readMe, err := NewReaderFromFilename(filename, *styles.Get("native"), formatters.TTY16m, nil)
 		if err != nil {
 			panic(err)
 		}
 
 		// Wait for the reader to finish
+		//revive:disable-next-line:empty-block
 		for !readMe.done.Load() {
 		}
 		if readMe.err != nil {
@@ -390,8 +368,8 @@ func BenchmarkReadLargeFile(b *testing.B) {
 	const largeSizeBytes = 35_000_000
 
 	// First, create it from something...
-	input_filename := getSamplesDir() + "/../m/pager.go"
-	contents, err := os.ReadFile(input_filename)
+	inputFilename := "pager.go"
+	contents, err := os.ReadFile(inputFilename)
 	if err != nil {
 		panic(err)
 	}
@@ -419,7 +397,7 @@ func BenchmarkReadLargeFile(b *testing.B) {
 
 	b.ResetTimer()
 	for n := 0; n < b.N; n++ {
-		readMe, err := NewReaderFromFilename(largeFileName, *styles.Get("native"), formatters.TTY16m)
+		readMe, err := NewReaderFromFilename(largeFileName, *styles.Get("native"), formatters.TTY16m, nil)
 		if err != nil {
 			panic(err)
 		}
@@ -436,8 +414,8 @@ func BenchmarkReadLargeFile(b *testing.B) {
 // Count lines in pager.go
 func BenchmarkCountLines(b *testing.B) {
 	// First, get some sample lines...
-	input_filename := getSamplesDir() + "/../m/pager.go"
-	contents, err := os.ReadFile(input_filename)
+	inputFilename := "pager.go"
+	contents, err := os.ReadFile(inputFilename)
 	if err != nil {
 		panic(err)
 	}
