@@ -3,6 +3,8 @@ package m
 import (
 	"fmt"
 
+	"github.com/walles/moar/m/linenumbers"
+	"github.com/walles/moar/m/textstyles"
 	"github.com/walles/moar/twin"
 )
 
@@ -14,7 +16,7 @@ const (
 )
 
 type renderedLine struct {
-	inputLineOneBased int
+	inputLine linenumbers.LineNumber
 
 	// If an input line has been wrapped into two, the part on the second line
 	// will have a wrapIndex of 1.
@@ -52,34 +54,12 @@ func (p *Pager) redraw(spinner string) overflowState {
 		// This happens when we're done
 		eofSpinner = "---"
 	}
-	spinnerLine := cellsFromString(_EofMarkerFormat+eofSpinner, nil).Cells
+	spinnerLine := textstyles.CellsFromString(_EofMarkerFormat+eofSpinner, nil).Cells
 	for column, cell := range spinnerLine {
 		p.screen.SetCell(column, lastUpdatedScreenLineNumber+1, cell)
 	}
 
-	switch p.mode {
-	case _Searching:
-		p.addSearchFooter()
-
-	case _NotFound:
-		p.setFooter("Not found: " + p.searchString)
-
-	case _GotoLine:
-		p.addGotoLineFooter()
-
-	case _Viewing:
-		helpText := "Press 'ESC' / 'q' to exit, '/' to search, '?' for help"
-		if p.isShowingHelp {
-			helpText = "Press 'ESC' / 'q' to exit help, '/' to search"
-		}
-
-		if p.ShowStatusBar {
-			p.setFooter(statusText + spinner + "  " + helpText)
-		}
-
-	default:
-		panic(fmt.Sprint("Unsupported pager mode: ", p.mode))
-	}
+	p.mode.drawFooter(statusText, spinner)
 
 	p.screen.Show()
 	return overflow
@@ -131,12 +111,23 @@ func (p *Pager) renderLines() ([]renderedLine, string, overflowState) {
 	wantedLineCount := p.visibleHeight()
 
 	screenOverflow := didFit
-	if p.lineNumberOneBased() > 1 {
+
+	var lineNumber linenumbers.LineNumber
+	if p.lineNumber() != nil {
+		lineNumber = *p.lineNumber()
+	} else {
+		// No lines to show, line number doesn't matter, pick anything. But we
+		// still want one so that we can get the status text from the reader
+		// below.
+		lineNumber = linenumbers.LineNumber{}
+	}
+
+	if !lineNumber.IsZero() {
 		// We're scrolled down, meaning everything is not visible on screen
 		screenOverflow = didOverflow
 	}
 
-	inputLines, readerOverflow := p.reader.GetLines(p.lineNumberOneBased(), wantedLineCount)
+	inputLines, readerOverflow := p.reader.GetLines(lineNumber, wantedLineCount)
 	if inputLines.lines == nil {
 		// Empty input, empty output
 		return []renderedLine{}, inputLines.statusText, didFit
@@ -149,7 +140,7 @@ func (p *Pager) renderLines() ([]renderedLine, string, overflowState) {
 	allLines := make([]renderedLine, 0)
 	for lineIndex, line := range inputLines.lines {
 
-		lineNumber := inputLines.firstLineOneBased + lineIndex
+		lineNumber := inputLines.firstLine.NonWrappingAdd(lineIndex)
 
 		rendering, lineOverflow := p.renderLine(line, lineNumber, p.scrollPosition.internalDontTouch)
 		if lineOverflow == didOverflow {
@@ -183,12 +174,12 @@ func (p *Pager) renderLines() ([]renderedLine, string, overflowState) {
 	// screen
 	firstVisibleIndex := -1 // Not found
 	for index, line := range allLines {
-		if p.lineNumberOneBased() == 0 {
+		if p.lineNumber() == nil {
 			// Expected zero lines but got some anyway, grab the first one!
 			firstVisibleIndex = index
 			break
 		}
-		if line.inputLineOneBased == p.lineNumberOneBased() && line.wrapIndex == p.deltaScreenLines() {
+		if line.inputLine == *p.lineNumber() && line.wrapIndex == p.deltaScreenLines() {
 			firstVisibleIndex = index
 			break
 		}
@@ -221,7 +212,7 @@ func (p *Pager) renderLines() ([]renderedLine, string, overflowState) {
 //
 // lineNumber and numberPrefixLength are required for knowing how much to
 // indent, and to (optionally) render the line number.
-func (p *Pager) renderLine(line *Line, lineNumber int, scrollPosition scrollPositionInternal) ([]renderedLine, overflowState) {
+func (p *Pager) renderLine(line *Line, lineNumber linenumbers.LineNumber, scrollPosition scrollPositionInternal) ([]renderedLine, overflowState) {
 	highlighted := line.HighlightedTokens(p.linePrefix, p.searchPattern, &lineNumber)
 	var wrapped [][]twin.Cell
 	overflow := didFit
@@ -250,9 +241,9 @@ func (p *Pager) renderLine(line *Line, lineNumber int, scrollPosition scrollPosi
 		}
 
 		rendered = append(rendered, renderedLine{
-			inputLineOneBased: lineNumber,
-			wrapIndex:         wrapIndex,
-			cells:             decorated,
+			inputLine: lineNumber,
+			wrapIndex: wrapIndex,
+			cells:     decorated,
 		})
 	}
 
@@ -269,7 +260,7 @@ func (p *Pager) renderLine(line *Line, lineNumber int, scrollPosition scrollPosi
 // * Line number, or leading whitespace for wrapped lines
 // * Scroll left indicator
 // * Scroll right indicator
-func (p *Pager) decorateLine(lineNumberToShow *int, contents []twin.Cell, scrollPosition scrollPositionInternal) ([]twin.Cell, overflowState) {
+func (p *Pager) decorateLine(lineNumberToShow *linenumbers.LineNumber, contents []twin.Cell, scrollPosition scrollPositionInternal) ([]twin.Cell, overflowState) {
 	width, _ := p.screen.Size()
 	newLine := make([]twin.Cell, 0, width)
 	numberPrefixLength := numberPrefixLength(p, scrollPosition)
@@ -315,21 +306,20 @@ func (p *Pager) decorateLine(lineNumberToShow *int, contents []twin.Cell, scroll
 // Generate a line number prefix of the given length.
 //
 // Can be empty or all-whitespace depending on parameters.
-func createLinePrefix(fileLineNumber *int, numberPrefixLength int) []twin.Cell {
+func createLinePrefix(lineNumber *linenumbers.LineNumber, numberPrefixLength int) []twin.Cell {
 	if numberPrefixLength == 0 {
 		return []twin.Cell{}
 	}
 
 	lineNumberPrefix := make([]twin.Cell, 0, numberPrefixLength)
-	if fileLineNumber == nil {
+	if lineNumber == nil {
 		for len(lineNumberPrefix) < numberPrefixLength {
 			lineNumberPrefix = append(lineNumberPrefix, twin.Cell{Rune: ' '})
 		}
 		return lineNumberPrefix
 	}
 
-	lineNumberString := formatNumber(uint(*fileLineNumber))
-	lineNumberString = fmt.Sprintf("%*s ", numberPrefixLength-1, lineNumberString)
+	lineNumberString := fmt.Sprintf("%*s ", numberPrefixLength-1, lineNumber.Format())
 	if len(lineNumberString) > numberPrefixLength {
 		panic(fmt.Errorf(
 			"lineNumberString <%s> longer than numberPrefixLength %d",
