@@ -6,10 +6,13 @@ import (
 	"testing"
 
 	"github.com/google/go-cmp/cmp"
+	"github.com/walles/moar/m/linenumbers"
 	"github.com/walles/moar/twin"
 	"gotest.tools/v3/assert"
 )
 
+//revive:disable:empty-block
+
 func testHorizontalCropping(t *testing.T, contents string, firstIndex int, lastIndex int, expected string, expectedOverflow overflowState) {
 	pager := NewPager(nil)
 	pager.ShowLineNumbers = false
@@ -19,7 +22,7 @@ func testHorizontalCropping(t *testing.T, contents string, firstIndex int, lastI
 	pager.scrollPosition = newScrollPosition("testHorizontalCropping")
 
 	lineContents := NewLine(contents)
-	screenLine, didOverflow := pager.renderLine(&lineContents, 0, pager.scrollPosition.internalDontTouch)
+	screenLine, didOverflow := pager.renderLine(&lineContents, linenumbers.LineNumber{}, pager.scrollPosition.internalDontTouch)
 	assert.Equal(t, rowToString(screenLine[0].cells), expected)
 	assert.Equal(t, didOverflow, expectedOverflow)
 }
@@ -61,7 +64,7 @@ func TestEmpty(t *testing.T) {
 	rendered, statusText, overflow := pager.renderScreenLines()
 	assert.Equal(t, len(rendered), 0)
 	assert.Equal(t, "test: <empty>", statusText)
-	assert.Equal(t, pager.lineNumberOneBased(), 0)
+	assert.Assert(t, pager.lineNumber() == nil)
 	assert.Equal(t, overflow, didFit)
 }
 
@@ -75,11 +78,11 @@ func TestSearchHighlight(t *testing.T) {
 		searchPattern: regexp.MustCompile("\""),
 	}
 
-	rendered, overflow := pager.renderLine(&line, 1, pager.scrollPosition.internalDontTouch)
+	rendered, overflow := pager.renderLine(&line, linenumbers.LineNumber{}, pager.scrollPosition.internalDontTouch)
 	assert.DeepEqual(t, []renderedLine{
 		{
-			inputLineOneBased: 1,
-			wrapIndex:         0,
+			inputLine: linenumbers.LineNumber{},
+			wrapIndex: 0,
 			cells: []twin.Cell{
 				{Rune: 'x', Style: twin.StyleDefault},
 				{Rune: '"', Style: twin.StyleDefault.WithAttr(twin.AttrReverse)},
@@ -87,7 +90,11 @@ func TestSearchHighlight(t *testing.T) {
 				{Rune: 'x', Style: twin.StyleDefault},
 			},
 		},
-	}, rendered, cmp.AllowUnexported(twin.Style{}), cmp.AllowUnexported(renderedLine{}))
+	}, rendered,
+		cmp.AllowUnexported(twin.Style{}),
+		cmp.AllowUnexported(renderedLine{}),
+		cmp.AllowUnexported(linenumbers.LineNumber{}),
+	)
 	assert.Equal(t, overflow, didFit)
 }
 
@@ -102,14 +109,14 @@ func TestOverflowDown(t *testing.T) {
 		reader: NewReaderFromText("test", "hej"),
 
 		// This value can be anything and should be clipped, that's what we're testing
-		scrollPosition: *scrollPositionFromLineNumber("TestOverflowDown", 42),
+		scrollPosition: *scrollPositionFromLineNumber("TestOverflowDown", linenumbers.LineNumberFromOneBased(42)),
 	}
 
 	rendered, statusText, overflow := pager.renderScreenLines()
 	assert.Equal(t, len(rendered), 1)
 	assert.Equal(t, "hej", rowToString(rendered[0]))
 	assert.Equal(t, "test: 1 line  100%", statusText)
-	assert.Equal(t, pager.lineNumberOneBased(), 1)
+	assert.Assert(t, pager.lineNumber().IsZero())
 	assert.Equal(t, pager.deltaScreenLines(), 0)
 	assert.Equal(t, overflow, didFit)
 }
@@ -131,14 +138,14 @@ func TestOverflowUp(t *testing.T) {
 	assert.Equal(t, len(rendered), 1)
 	assert.Equal(t, "hej", rowToString(rendered[0]))
 	assert.Equal(t, "test: 1 line  100%", statusText)
-	assert.Equal(t, pager.lineNumberOneBased(), 1)
+	assert.Assert(t, pager.lineNumber().IsZero())
 	assert.Equal(t, pager.deltaScreenLines(), 0)
 	assert.Equal(t, overflow, didFit)
 }
 
 func TestWrapping(t *testing.T) {
-	reader := NewReaderFromStream("",
-		strings.NewReader("first line\nline two will be wrapped\nhere's the last line"))
+	reader := NewReaderFromText("",
+		"first line\nline two will be wrapped\nhere's the last line")
 	pager := NewPager(reader)
 	pager.screen = twin.NewFakeScreen(40, 40)
 
