@@ -4,7 +4,6 @@ import (
 	"flag"
 	"fmt"
 	"io"
-	"math"
 	"os"
 	"os/exec"
 	"path/filepath"
@@ -15,11 +14,14 @@ import (
 
 	"github.com/alecthomas/chroma/v2"
 	"github.com/alecthomas/chroma/v2/formatters"
+	"github.com/alecthomas/chroma/v2/lexers"
 	"github.com/alecthomas/chroma/v2/styles"
 	log "github.com/sirupsen/logrus"
 	"golang.org/x/term"
 
 	"github.com/walles/moar/m"
+	"github.com/walles/moar/m/linenumbers"
+	"github.com/walles/moar/m/textstyles"
 	"github.com/walles/moar/twin"
 )
 
@@ -156,11 +158,28 @@ func printProblemsHeader() {
 	fmt.Fprintln(os.Stderr, "NumCPU  :", runtime.NumCPU())
 }
 
+func parseLexerOption(lexerOption string) (chroma.Lexer, error) {
+	byMimeType := lexers.MatchMimeType(lexerOption)
+	if byMimeType != nil {
+		return byMimeType, nil
+	}
+
+	// Use Chroma's built-in fuzzy lexer picker
+	lexer := lexers.Get(lexerOption)
+	if lexer != nil {
+		return lexer, nil
+	}
+
+	return nil, fmt.Errorf(
+		"Look here for inspiration: https://github.com/alecthomas/chroma/tree/master/lexers/embedded",
+	)
+}
+
 func parseStyleOption(styleOption string) (chroma.Style, error) {
 	style, ok := styles.Registry[styleOption]
 	if !ok {
 		return *styles.Fallback, fmt.Errorf(
-			"Pick a style from here: https://xyproto.github.io/splash/docs/longer/all.html\n")
+			"Pick a style from here: https://xyproto.github.io/splash/docs/longer/all.html")
 	}
 
 	return *style, nil
@@ -187,7 +206,7 @@ func parseColorsOption(colorsOption string) (twin.ColorType, error) {
 	}
 
 	var noColor twin.ColorType
-	return noColor, fmt.Errorf("Valid counts are 8, 16, 256, 16M or auto.")
+	return noColor, fmt.Errorf("Valid counts are 8, 16, 256, 16M or auto")
 }
 
 func parseStatusBarStyle(styleOption string) (m.StatusBarOption, error) {
@@ -201,15 +220,15 @@ func parseStatusBarStyle(styleOption string) (m.StatusBarOption, error) {
 		return m.STATUSBAR_STYLE_BOLD, nil
 	}
 
-	return 0, fmt.Errorf("good ones are inverse, plain and bold")
+	return 0, fmt.Errorf("Good ones are inverse, plain and bold")
 }
 
-func parseUnprintableStyle(styleOption string) (m.UnprintableStyle, error) {
+func parseUnprintableStyle(styleOption string) (textstyles.UnprintableStyleT, error) {
 	if styleOption == "highlight" {
-		return m.UNPRINTABLE_STYLE_HIGHLIGHT, nil
+		return textstyles.UnprintableStyleHighlight, nil
 	}
 	if styleOption == "whitespace" {
-		return m.UNPRINTABLE_STYLE_WHITESPACE, nil
+		return textstyles.UnprintableStyleWhitespace, nil
 	}
 
 	return 0, fmt.Errorf("Good ones are highlight or whitespace")
@@ -233,7 +252,7 @@ func parseShiftAmount(shiftAmount string) (uint, error) {
 	}
 
 	if value < 1 {
-		return 0, fmt.Errorf("Shift amount must be at least 1, was %d", value)
+		return 0, fmt.Errorf("Shift amount must be at least 1")
 	}
 
 	// Let's add an upper bound as well if / when requested
@@ -259,7 +278,7 @@ func pumpToStdout(inputFilename *string) error {
 		// If we get both redirected stdin and an input filename, we must prefer
 		// to copy the file, because that's how less works. That's why we go for
 		// the filename first.
-		inputFile, err := os.Open(*inputFilename)
+		inputFile, err := m.ZOpen(*inputFilename)
 		if err != nil {
 			return fmt.Errorf("Failed to open %s: %w", *inputFilename, err)
 		}
@@ -309,8 +328,8 @@ func tryOpen(filename string) error {
 // Parses an argument like "+123" anywhere on the command line into a one-based
 // line number, and returns the remaining args.
 //
-// Returns 0 on no target line number specified.
-func getTargetLineNumberOneBased(args []string) (int, []string) {
+// Returns nil on no target line number specified.
+func getTargetLineNumber(args []string) (*linenumbers.LineNumber, []string) {
 	for i, arg := range args {
 		if !strings.HasPrefix(arg, "+") {
 			continue
@@ -333,10 +352,11 @@ func getTargetLineNumberOneBased(args []string) (int, []string) {
 		remainingArgs = append(remainingArgs, args[:i]...)
 		remainingArgs = append(remainingArgs, args[i+1:]...)
 
-		return int(lineNumber), remainingArgs
+		returnMe := linenumbers.LineNumberFromOneBased(int(lineNumber))
+		return &returnMe, remainingArgs
 	}
 
-	return 0, args
+	return nil, args
 }
 
 func main() {
@@ -381,6 +401,9 @@ func main() {
 	style := flagSetFunc(flagSet,
 		"style", *styles.Registry["native"],
 		"Highlighting style from https://xyproto.github.io/splash/docs/longer/all.html", parseStyleOption)
+	lexer := flagSetFunc(flagSet,
+		"lang", nil,
+		"File contents, used for highlighting. Mime type or file extension (\"html\"). Default is to guess by filename.", parseLexerOption)
 
 	defaultFormatter, err := parseColorsOption("auto")
 	if err != nil {
@@ -395,7 +418,7 @@ func main() {
 	noClearOnExit := flagSet.Bool("no-clear-on-exit", false, "Retain screen contents when exiting moar")
 	statusBarStyle := flagSetFunc(flagSet, "statusbar", m.STATUSBAR_STYLE_INVERSE,
 		"Status bar style: inverse, plain or bold", parseStatusBarStyle)
-	unprintableStyle := flagSetFunc(flagSet, "render-unprintable", m.UNPRINTABLE_STYLE_HIGHLIGHT,
+	unprintableStyle := flagSetFunc(flagSet, "render-unprintable", textstyles.UnprintableStyleHighlight,
 		"How unprintable characters are rendered: highlight or whitespace", parseUnprintableStyle)
 	scrollLeftHint := flagSetFunc(flagSet, "scroll-left-hint",
 		twin.NewCell('<', twin.StyleDefault.WithAttr(twin.AttrReverse)),
@@ -421,7 +444,7 @@ func main() {
 		flags = append(strings.Fields(moarEnv), flags...)
 	}
 
-	targetLineNumberOneBased, remainingArgs := getTargetLineNumberOneBased(flags)
+	targetLineNumber, remainingArgs := getTargetLineNumber(flags)
 
 	err = flagSet.Parse(remainingArgs)
 	if err != nil {
@@ -524,10 +547,10 @@ func main() {
 	var reader *m.Reader
 	if stdinIsRedirected {
 		// Display input pipe contents
-		reader = m.NewReaderFromStream("", os.Stdin)
+		reader = m.NewReaderFromStream("", os.Stdin, *style, formatter, *lexer)
 	} else {
 		// Display the input file contents
-		reader, err = m.NewReaderFromFilename(*inputFilename, *style, formatter)
+		reader, err = m.NewReaderFromFilename(*inputFilename, *style, formatter, *lexer)
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "ERROR: %v\n", err)
 			os.Exit(1)
@@ -546,9 +569,10 @@ func main() {
 	pager.ScrollRightHint = *scrollRightHint
 	pager.SideScrollAmount = int(*shift)
 
-	pager.TargetLineNumberOneBased = targetLineNumberOneBased
-	if *follow && pager.TargetLineNumberOneBased == 0 {
-		pager.TargetLineNumberOneBased = math.MaxInt
+	pager.TargetLineNumber = targetLineNumber
+	if *follow && pager.TargetLineNumber == nil {
+		reallyHigh := linenumbers.LineNumberMax()
+		pager.TargetLineNumber = &reallyHigh
 	}
 
 	startPaging(pager, screen, style, &formatter)
